import { Injectable, Inject } from '@angular/core';
import { Router } from '@angular/router';

import { of, throwError, EMPTY as empty } from 'rxjs';
import {
  catchError,
  exhaustMap,
  map,
  tap,
  take,
  switchMap,
  mergeMap,
  skip,
  concatMap
} from 'rxjs/operators';

import { Store } from '@ngrx/store';
import { Actions, ofType, createEffect } from '@ngrx/effects';

import {
  PiService,
  Login2Response,
  ResponseStatus
} from '@ucap-webmessenger/pi';
import { NativeService, UCAP_NATIVE_SERVICE } from '@ucap-webmessenger/native';
import {
  DialogService,
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult
} from '@ucap-webmessenger/ui';

import {
  loginSuccess,
  loginFailure,
  loginRedirect,
  logout,
  logoutConfirmation,
  logoutConfirmationDismiss,
  webLogin,
  webLoginSuccess,
  webLoginFailure
} from './actions';
import {
  LoginInfo,
  EnvironmentsInfo,
  KEY_ENVIRONMENTS_INFO
} from '../../../types';
import { AppAuthenticationService } from '../../../services/authentication.service';
import { NGXLogger } from 'ngx-logger';
import {
  PublicApiService,
  VersionInfo2Response
} from '@ucap-webmessenger/api-public';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';

import * as VersionInfoStore from '@app/store/setting/version-info';
import { ProtocolService } from '@ucap-webmessenger/protocol';
import { InnerProtocolService } from '@ucap-webmessenger/protocol-inner';
import {
  AuthenticationProtocolService,
  SSOMode,
  LoginResponse
} from '@ucap-webmessenger/protocol-authentication';

import CryptoJS from 'crypto-js';

@Injectable()
export class Effects {
  // webLogin$ = createEffect(() =>
  //   this.actions$.pipe(
  //     ofType(webLogin),
  //     map(action => action),
  //     exhaustMap((params: { loginInfo: LoginInfo; rememberMe: boolean }) =>
  //       this.piService
  //         .login2({
  //           loginId: params.loginInfo.loginId,
  //           loginPw: params.loginInfo.loginPw,
  //           companyCode: params.loginInfo.companyCode
  //         })
  //         .pipe(
  //           map((res: Login2Response) => {
  //             if ('success' !== res.status.toLowerCase()) {
  //               return webLoginFailure({ error: 'Failed' });
  //             } else {
  //               return webLoginSuccess({
  //                 loginInfo: params.loginInfo,
  //                 rememberMe: params.rememberMe,
  //                 login2Response: res
  //               });
  //             }
  //           }),
  //           catchError(error => of(webLoginFailure({ error })))
  //         )
  //     )
  //   )
  // );
  webLogin$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(webLogin),
        map(action => action),
        switchMap(
          (loginParams: { loginInfo: LoginInfo; rememberMe: boolean }) => {
            const environmentsInfo = this.sessionStorageService.get<
              EnvironmentsInfo
            >(KEY_ENVIRONMENTS_INFO);

            let login2Res: Login2Response;
            let loginRes: LoginResponse;
            let versionInfo2Res: VersionInfo2Response;
            let encLoginPw: string;

            return this.piService
              .login2({
                loginId: loginParams.loginInfo.loginId,
                loginPw: loginParams.loginInfo.loginPw,
                companyCode: loginParams.loginInfo.companyCode
              })
              .pipe(
                concatMap(res => {
                  console.log('tap');
                  if ('success' !== res.status.toLowerCase()) {
                    return throwError('login2 failed');
                  }
                  login2Res = res;
                  return of(true);
                }),
                mergeMap(dd => {
                  console.log('checkForUpdates');
                  return this.nativeService.checkForUpdates();
                }),
                concatMap(update => {
                  if (update) {
                    return throwError('update process');
                  }
                  encLoginPw = CryptoJS.enc.Hex.stringify(
                    CryptoJS.SHA256(loginParams.loginInfo.loginPw)
                  );
                  return of(true);
                }),
                mergeMap(() =>
                  this.publicApiService.versionInfo2({
                    deviceType: environmentsInfo.deviceType,
                    companyGroupType: loginParams.loginInfo.companyGroupType,
                    companyCode: loginParams.loginInfo.companyCode,
                    loginId: loginParams.loginInfo.loginId
                  })
                ),
                map(res => {
                  this.store.dispatch(VersionInfoStore.fetchSuccess(res));
                  versionInfo2Res = res;
                  return of(true);
                }),
                mergeMap(() =>
                  this.protocolService.connect(versionInfo2Res.serverIp)
                ),
                mergeMap(() => this.innerProtocolService.conn({})),
                mergeMap(() =>
                  this.authenticationProtocolService.login({
                    loginId: loginParams.loginInfo.loginId,
                    loginPw: encLoginPw,
                    deviceType: environmentsInfo.deviceType,
                    deviceId: ' ',
                    token: '',
                    localeCode: loginParams.loginInfo.localeCode,
                    pushId: ' ',
                    companyCode: loginParams.loginInfo.companyCode,
                    passwordEncodingType: 1,
                    clientVersion: '',
                    reconnect: false,
                    ip: 'localhost',
                    hostName: '',
                    ssoMode: SSOMode.AUTH,
                    userSpecificInformation: 'PRO_000482',
                    productId: 'PRO_000482',
                    productName: 'EZMessenger'
                  })
                ),
                exhaustMap(async res => {
                  if (res.privateInformationAgree) {
                    return of(null);
                  }

                  loginRes = res;

                  const privacyTotalUrl = this.piService.privacyTotalUrl({
                    companyCode: res.companyCode,
                    userSeq: res.userSeq,
                    token: res.tokenString,
                    deviceType: environmentsInfo.deviceType,
                    localeCode: loginParams.loginInfo.localeCode,
                    textOnly: 'true'
                  });

                  const result = await this.dialogService.open<
                    ConfirmDialogComponent,
                    ConfirmDialogData,
                    ConfirmDialogResult
                  >(ConfirmDialogComponent, {
                    width: '100%',
                    height: '500px',
                    disableClose: true,
                    data: {
                      title: '개인정보 동의',
                      html: `<iframe id="ifm_privacy" src="${privacyTotalUrl}" style="width: 100%;height: 300px;" />`
                    }
                  });

                  if (!result.choice) {
                    console.log('privacy error');
                    throw new Error('privacy error');
                  }

                  return of(true);
                }),
                exhaustMap(proceed => {
                  if (null === proceed) {
                    return of(null);
                  }

                  return this.piService.userTermsAction({
                    userSeq: loginRes.userSeq,
                    token: loginRes.tokenString,
                    deviceType: environmentsInfo.deviceType
                  });
                }),
                exhaustMap(async () => {
                  this.logger.debug(
                    'loginRes.passwordExpired',
                    loginRes.passwordExpired
                  );
                  if (!loginRes.passwordExpired) {
                    return of(null);
                  }

                  const privacyTotalUrl = this.piService.privacyTotalUrl({
                    companyCode: loginRes.companyCode,
                    userSeq: loginRes.userSeq,
                    token: loginRes.tokenString,
                    deviceType: environmentsInfo.deviceType,
                    localeCode: loginParams.loginInfo.localeCode,
                    textOnly: 'true'
                  });

                  const result = await this.dialogService.open<
                    ConfirmDialogComponent,
                    ConfirmDialogData,
                    ConfirmDialogResult
                  >(ConfirmDialogComponent, {
                    width: '100%',
                    height: '500px',
                    disableClose: true,
                    data: {
                      title: '패스워드 변경',
                      html: `<iframe id="ifm_privacy" src="${privacyTotalUrl}" style="width: 100%;height: 300px;" />`
                    }
                  });

                  if (!result.choice) {
                    return throwError('privacy error');
                  }

                  return of(true);
                }),
                mergeMap(proceed => {
                  // 패스워드 변경
                  if (null === proceed) {
                    return of(null);
                  }

                  return this.piService.userTermsAction({
                    userSeq: loginRes.userSeq,
                    token: loginRes.tokenString,
                    deviceType: environmentsInfo.deviceType
                  });
                }),

                map(res => {
                  this.appAuthenticationService.login(
                    loginParams.loginInfo,
                    loginParams.rememberMe
                  );
                  this.store.dispatch(
                    loginSuccess({
                      loginInfo: loginRes
                    })
                  );
                  this.router.navigate(['/messenger']);
                }),
                catchError(error => {
                  return throwError(error);
                })
              );
          }
        ),
        catchError(error => {
          this.store.dispatch(webLoginFailure({ error: error.message }));
          return of(error);
        })
      ),
    { dispatch: false }
  );

  webLoginSuccess$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(webLoginSuccess),
        switchMap(params =>
          this.nativeService.checkForUpdates().pipe(
            map((update: boolean) => {
              if (!update) {
                this.appAuthenticationService.login(
                  params.loginInfo,
                  params.rememberMe
                );
                this.router.navigate(['/messenger']);
              }
            }),
            catchError(error => of(error))
          )
        )
      ),
    { dispatch: false }
  );

  loginRedirect$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(loginRedirect),
        tap(authed => {
          this.router.navigate(['/account/login']);
        })
      ),
    { dispatch: false }
  );

  logout$ = createEffect(() =>
    this.actions$.pipe(
      ofType(logout),
      map(action => action),
      map(() => {
        this.appAuthenticationService.logout();
        return loginRedirect();
      })
    )
  );

  logoutConfirmation$ = createEffect(() =>
    this.actions$.pipe(
      ofType(logoutConfirmation),
      exhaustMap(async () => {
        const result = await this.dialogService.open<
          ConfirmDialogComponent,
          ConfirmDialogData,
          ConfirmDialogResult
        >(ConfirmDialogComponent, {
          width: '220px',
          data: {
            title: 'Logout',
            message: 'Logout ?'
          }
        });

        return result.choice;
      }),
      map(result => (result ? logout() : logoutConfirmationDismiss()))
    )
  );

  constructor(
    private actions$: Actions,
    private router: Router,
    private store: Store<any>,
    private piService: PiService,
    private publicApiService: PublicApiService,
    private protocolService: ProtocolService,
    private innerProtocolService: InnerProtocolService,
    private authenticationProtocolService: AuthenticationProtocolService,
    private appAuthenticationService: AppAuthenticationService,
    private dialogService: DialogService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private sessionStorageService: SessionStorageService,
    private logger: NGXLogger
  ) {}
}
